package java.lang.invoke;

import sun.misc.VM;
import java.lang.reflect.Modifier;
import sun.security.util.SecurityConstants;
import sun.reflect.misc.ReflectUtil;
import java.lang.reflect.Field;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import sun.invoke.util.VerifyAccess;
import java.util.concurrent.ConcurrentHashMap;
import java.lang.reflect.ReflectPermission;
import java.util.Iterator;
import java.util.List;
import sun.invoke.util.ValueConversions;
import sun.invoke.util.Wrapper;
import java.util.BitSet;
import java.util.Arrays;
import java.lang.reflect.Member;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.security.Permission;

public class MethodHandles
{
    private static final MemberName.Factory IMPL_NAMES;
    private static final Permission ACCESS_PERMISSION;
    private static final MethodHandle[] IDENTITY_MHS;
    private static final MethodHandle[] ZERO_MHS;
    static final /* synthetic */ boolean $assertionsDisabled;
    
    private MethodHandles() {
    }
    
    @CallerSensitive
    public static MethodHandles.Lookup lookup() {
        return new MethodHandles.Lookup(Reflection.getCallerClass());
    }
    
    public static MethodHandles.Lookup publicLookup() {
        return MethodHandles.Lookup.PUBLIC_LOOKUP;
    }
    
    public static <T extends Member> T reflectAs(Class<T> clazz, MethodHandle methodHandle) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(MethodHandles.ACCESS_PERMISSION);
        }
        MethodHandles.Lookup impl_LOOKUP = MethodHandles.Lookup.IMPL_LOOKUP;
        return impl_LOOKUP.revealDirect(methodHandle).reflectAs(clazz, impl_LOOKUP);
    }
    
    public static MethodHandle arrayElementGetter(Class<?> clazz) throws IllegalArgumentException {
        return MethodHandleImpl.makeArrayElementAccessor(clazz, false);
    }
    
    public static MethodHandle arrayElementSetter(Class<?> clazz) throws IllegalArgumentException {
        return MethodHandleImpl.makeArrayElementAccessor(clazz, true);
    }
    
    public static MethodHandle spreadInvoker(MethodType spreaderType, int i) {
        if (i < 0 || i > spreaderType.parameterCount()) {
            throw MethodHandleStatics.newIllegalArgumentException("bad argument count", Integer.valueOf(i));
        }
        spreaderType = spreaderType.asSpreaderType(Object[].class, spreaderType.parameterCount() - i);
        return spreaderType.invokers().spreadInvoker(i);
    }
    
    public static MethodHandle exactInvoker(MethodType methodType) {
        return methodType.invokers().exactInvoker();
    }
    
    public static MethodHandle invoker(MethodType methodType) {
        return methodType.invokers().genericInvoker();
    }
    
    static MethodHandle basicInvoker(MethodType methodType) {
        return methodType.invokers().basicInvoker();
    }
    
    public static MethodHandle explicitCastArguments(MethodHandle methodHandle, MethodType methodType) {
        explicitCastArgumentsChecks(methodHandle, methodType);
        MethodType type = methodHandle.type();
        if (type == methodType) {
            return methodHandle;
        }
        if (type.explicitCastEquivalentToAsType(methodType)) {
            return methodHandle.asFixedArity().asType(methodType);
        }
        return MethodHandleImpl.makePairwiseConvert(methodHandle, methodType, false);
    }
    
    private static void explicitCastArgumentsChecks(MethodHandle obj, MethodType obj2) {
        if (obj.type().parameterCount() != obj2.parameterCount()) {
            throw new WrongMethodTypeException("cannot explicitly cast " + obj + " to " + obj2);
        }
        return;
    }
    
    public static MethodHandle permuteArguments(MethodHandle methodHandle, MethodType methodType, int... array) {
        array = (int[])array.clone();
        MethodType methodType2 = methodHandle.type();
        permuteArgumentChecks(array, methodType, methodType2);
        BoundMethodHandle rebind = methodHandle.rebind();
        LambdaForm lambdaForm = rebind.form;
        int parameterCount = methodType.parameterCount();
        int firstDupOrDrop;
        while ((firstDupOrDrop = findFirstDupOrDrop(array, parameterCount)) != 0) {
            if (firstDupOrDrop > 0) {
                firstDupOrDrop = (firstDupOrDrop = firstDupOrDrop);
                int n4 = array[firstDupOrDrop];
                boolean n5 = false;
                int n6;
                while ((n6 = array[--firstDupOrDrop]) != n4) {
                    if (n4 <= n6) {
                        continue;
                    }
                    n5 = true;
                }
                if (n5 == 0) {
                    firstDupOrDrop = (firstDupOrDrop = firstDupOrDrop);
                }
                lambdaForm = lambdaForm.editor().dupArgumentForm(1 + firstDupOrDrop, 1 + firstDupOrDrop);
                assert array[firstDupOrDrop] == array[firstDupOrDrop];
                methodType2 = methodType2.dropParameterTypes(firstDupOrDrop, firstDupOrDrop + 1);
                int n7 = firstDupOrDrop + 1;
                System.arraycopy(array, n7, array, firstDupOrDrop, array.length - n7);
                array = Arrays.copyOf(array, array.length - 1);
            }
            else {
                int n;
                int n2;
                for (n = ~firstDupOrDrop, n2 = 0; n2 < array.length && array[n2] < n; ++n2) {}
                Class<Object> parameterType = methodType.parameterType(n);
                lambdaForm = lambdaForm.editor().addArgumentForm(1 + n2, LambdaForm.BasicType.basicType(parameterType));
                methodType2 = methodType2.insertParameterTypes(n2, new Class[] { parameterType });
                int n3 = n2 + 1;
                array = Arrays.copyOf(array, array.length + 1);
                System.arraycopy(array, n2, array, n3, array.length - n3);
                array[n2] = n;
            }
            if (MethodHandles.$assertionsDisabled || permuteArgumentChecks(array, methodType, methodType2)) {
                continue;
            }
            throw new AssertionError();
        }
        assert array.length == parameterCount;
        LambdaForm permuteArgumentsForm = lambdaForm.editor().permuteArgumentsForm(1, array);
        if (methodType == rebind.type() && permuteArgumentsForm == rebind.internalForm()) {
            return rebind;
        }
        return rebind.copyWith(methodType, permuteArgumentsForm);
    }
    
    private static int findFirstDupOrDrop(int[] array, int nbits) {
        if (nbits < 63) {
            long n2 = 0L;
            int j = 0;
            while (j < array.length) {
                int n3 = array[j];
                if (n3 >= nbits) {
                    return array.length;
                }
                long n4 = 1L << n3;
                if ((n2 & n4) != 0x0L) {
                    return j;
                }
                n2 |= n4;
                ++j;
            }
            if (n2 == (1L << nbits) - 1L) {
                assert Long.numberOfTrailingZeros(Long.lowestOneBit(~n2)) == nbits;
                return 0;
            }
            else {
                int numberOfTrailingZeros = Long.numberOfTrailingZeros(Long.lowestOneBit(~n2));
                assert numberOfTrailingZeros <= nbits;
                if (numberOfTrailingZeros == nbits) {
                    return 0;
                }
                return ~numberOfTrailingZeros;
            }
        }
        else {
            BitSet set = new BitSet(nbits);
            int i = 0;
            while (i < array.length) {
                int n = array[i];
                if (n >= nbits) {
                    return array.length;
                }
                if (set.get(n)) {
                    return i;
                }
                set.set(n);
                ++i;
            }
            int nextClearBit = set.nextClearBit(0);
            assert nextClearBit <= nbits;
            if (nextClearBit == nbits) {
                return 0;
            }
            return ~nextClearBit;
        }
    }
    
    private static boolean permuteArgumentChecks(int[] a, MethodType methodType, MethodType methodType2) {
        if (methodType.returnType() != methodType2.returnType()) {
            throw MethodHandleStatics.newIllegalArgumentException("return types do not match", methodType2, methodType);
        }
        if (a.length == methodType2.parameterCount()) {
            int parameterCount = methodType.parameterCount();
            boolean n = false;
            int i = 0;
            while (i < a.length) {
                int n2 = a[i];
                if (n2 < 0 || n2 >= parameterCount) {
                    n = true;
                    break;
                }
                if (methodType.parameterType(n2) != methodType2.parameterType(i)) {
                    throw MethodHandleStatics.newIllegalArgumentException("parameter types do not match after reorder", methodType2, methodType);
                }
                ++i;
            }
            if (n == 0) {
                return true;
            }
        }
        throw MethodHandleStatics.newIllegalArgumentException("bad reorder array: " + Arrays.toString(a));
    }
    
    public static MethodHandle constant(Class<?> clazz, Object convert) {
        if (clazz.isPrimitive()) {
            if (clazz == Void.TYPE) {
                throw MethodHandleStatics.newIllegalArgumentException("void type");
            }
            Wrapper forPrimitiveType = Wrapper.forPrimitiveType(clazz);
            convert = forPrimitiveType.convert(convert, clazz);
            if (forPrimitiveType.zero().equals(convert)) {
                return zero(forPrimitiveType, clazz);
            }
            return insertArguments(identity(clazz), 0, new Object[] { convert });
        }
        else {
            if (convert == null) {
                return zero(Wrapper.OBJECT, clazz);
            }
            return identity(clazz).bindTo(convert);
        }
    }
    
    public static MethodHandle identity(Class<?> type) {
        Wrapper wrapper = type.isPrimitive() ? Wrapper.forPrimitiveType(type) : Wrapper.OBJECT;
        int ordinal = wrapper.ordinal();
        MethodHandle setCachedMethodHandle = MethodHandles.IDENTITY_MHS[ordinal];
        if (setCachedMethodHandle == null) {
            setCachedMethodHandle = setCachedMethodHandle(MethodHandles.IDENTITY_MHS, ordinal, makeIdentity(wrapper.primitiveType()));
        }
        if (setCachedMethodHandle.type().returnType() == type) {
            return setCachedMethodHandle;
        }
        assert wrapper == Wrapper.OBJECT;
        return makeIdentity(type);
    }
    
    private static MethodHandle makeIdentity(Class<?> clazz) {
        return MethodHandleImpl.makeIntrinsic(MethodType.methodType(clazz, clazz), LambdaForm.identityForm(LambdaForm.BasicType.basicType(clazz)), MethodHandleImpl.Intrinsic.IDENTITY);
    }
    
    private static MethodHandle zero(Wrapper wrapper, Class<?> clazz) {
        int ordinal = wrapper.ordinal();
        MethodHandle setCachedMethodHandle = MethodHandles.ZERO_MHS[ordinal];
        if (setCachedMethodHandle == null) {
            setCachedMethodHandle = setCachedMethodHandle(MethodHandles.ZERO_MHS, ordinal, makeZero(wrapper.primitiveType()));
        }
        if (setCachedMethodHandle.type().returnType() == clazz) {
            return setCachedMethodHandle;
        }
        assert wrapper == Wrapper.OBJECT;
        return makeZero(clazz);
    }
    
    private static MethodHandle makeZero(Class<?> clazz) {
        return MethodHandleImpl.makeIntrinsic(MethodType.methodType(clazz), LambdaForm.zeroForm(LambdaForm.BasicType.basicType(clazz)), MethodHandleImpl.Intrinsic.ZERO);
    }
    
    private static synchronized MethodHandle setCachedMethodHandle(MethodHandle[] array, int n, MethodHandle methodHandle) {
        MethodHandle methodHandle2 = array[n];
        if (methodHandle2 != null) {
            return methodHandle2;
        }
        return array[n] = methodHandle;
    }
    
    public static MethodHandle insertArguments(MethodHandle methodHandle, int n, Object... array) {
        int length = array.length;
        Class[] insertArgumentsChecks = insertArgumentsChecks(methodHandle, length, n);
        if (length == 0) {
            return methodHandle;
        }
        BoundMethodHandle boundMethodHandle = methodHandle.rebind();
        for (int i = 0; i < length; ++i) {
            Object obj = array[i];
            Class<Object> clazz = insertArgumentsChecks[n + i];
            if (clazz.isPrimitive()) {
                boundMethodHandle = insertArgumentPrimitive(boundMethodHandle, n, clazz, obj);
            }
            else {
                boundMethodHandle = boundMethodHandle.bindArgumentL(n, clazz.cast(obj));
            }
        }
        return boundMethodHandle;
    }
    
    private static BoundMethodHandle insertArgumentPrimitive(BoundMethodHandle boundMethodHandle, int n, Class<?> clazz, Object convert) {
        Wrapper forPrimitiveType = Wrapper.forPrimitiveType(clazz);
        convert = forPrimitiveType.convert(convert, clazz);
        switch (forPrimitiveType) {
            case INT: {
                return boundMethodHandle.bindArgumentI(n, (int)convert);
            }
            case LONG: {
                return boundMethodHandle.bindArgumentJ(n, (long)convert);
            }
            case FLOAT: {
                return boundMethodHandle.bindArgumentF(n, (float)convert);
            }
            case DOUBLE: {
                return boundMethodHandle.bindArgumentD(n, (double)convert);
            }
            default: {
                return boundMethodHandle.bindArgumentI(n, ValueConversions.widenSubword(convert));
            }
        }
    }
    
    private static Class<?>[] insertArgumentsChecks(MethodHandle methodHandle, int n, int n2) throws RuntimeException {
        MethodType type = methodHandle.type();
        int n3 = type.parameterCount() - n;
        if (n3 < 0) {
            throw MethodHandleStatics.newIllegalArgumentException("too many values to insert");
        }
        if (n2 < 0 || n2 > n3) {
            throw MethodHandleStatics.newIllegalArgumentException("no argument type to append");
        }
        return type.ptypes();
    }
    
    public static MethodHandle dropArguments(MethodHandle methodHandle, int n, List<Class<?>> list) {
        List<Class<?>> copyTypes = copyTypes(list);
        MethodType type = methodHandle.type();
        int dropArgumentChecks = dropArgumentChecks(type, n, copyTypes);
        MethodType insertParameterTypes = type.insertParameterTypes(n, copyTypes);
        if (dropArgumentChecks == 0) {
            return methodHandle;
        }
        BoundMethodHandle rebind = methodHandle.rebind();
        LambdaForm lambdaForm = rebind.form;
        int n2 = 1 + n;
        Iterator<Object> iterator = copyTypes.iterator();
        while (iterator.hasNext()) {
            lambdaForm = lambdaForm.editor().addArgumentForm(n2++, LambdaForm.BasicType.basicType((Class)iterator.next()));
        }
        return rebind.copyWith(insertParameterTypes, lambdaForm);
    }
    
    private static List<Class<?>> copyTypes(List<Class<?>> list) {
        Object[] array = list.toArray();
        return Arrays.asList(Arrays.copyOf(array, array.length, Class[].class));
    }
    
    private static int dropArgumentChecks(MethodType methodType, int i, List<Class<?>> list) {
        int size = list.size();
        MethodType.checkSlotCount(size);
        int parameterCount = methodType.parameterCount();
        int j = parameterCount + size;
        if (i < 0 || i > parameterCount) {
            throw MethodHandleStatics.newIllegalArgumentException("no argument type to remove" + Arrays.asList(new Object[] { methodType, Integer.valueOf(i), list, Integer.valueOf(j), Integer.valueOf(parameterCount) }));
        }
        return size;
    }
    
    public static MethodHandle dropArguments(MethodHandle methodHandle, int n, Class<?>... a) {
        return dropArguments(methodHandle, n, Arrays.asList(a));
    }
    
    public static MethodHandle filterArguments(MethodHandle filterArgument, int n, MethodHandle... array) {
        filterArgumentsCheckArity(filterArgument, n, array);
        int n2 = n - 1;
        for (MethodHandle methodHandle : array) {
            ++n2;
            if (methodHandle != null) {
                filterArgument = filterArgument(filterArgument, n2, methodHandle);
            }
        }
        return filterArgument;
    }
    
    static MethodHandle filterArgument(MethodHandle methodHandle, int n, MethodHandle methodHandle2) {
        filterArgumentChecks(methodHandle, n, methodHandle2);
        MethodType type = methodHandle.type();
        MethodType type2 = methodHandle2.type();
        BoundMethodHandle rebind = methodHandle.rebind();
        Class<Object> parameterType = type2.parameterType(0);
        return rebind.copyWithExtendL(type.changeParameterType(n, parameterType), rebind.editor().filterArgumentForm(1 + n, LambdaForm.BasicType.basicType(parameterType)), methodHandle2);
    }
    
    private static void filterArgumentsCheckArity(MethodHandle methodHandle, int n, MethodHandle[] array) {
        if (n + array.length > methodHandle.type().parameterCount()) {
            throw MethodHandleStatics.newIllegalArgumentException("too many filters");
        }
        return;
    }
    
    private static void filterArgumentChecks(MethodHandle methodHandle, int n, MethodHandle methodHandle2) throws RuntimeException {
        MethodType type = methodHandle.type();
        MethodType type2 = methodHandle2.type();
        if (type2.parameterCount() != 1 || type2.returnType() != type.parameterType(n)) {
            throw MethodHandleStatics.newIllegalArgumentException("target and filter types do not match", type, type2);
        }
        return;
    }
    
    public static MethodHandle collectArguments(MethodHandle methodHandle, int n, MethodHandle methodHandle2) {
        MethodType collectArgumentsChecks = collectArgumentsChecks(methodHandle, n, methodHandle2);
        MethodType type = methodHandle2.type();
        BoundMethodHandle rebind = methodHandle.rebind();
        if (type.returnType().isArray() && methodHandle2.intrinsicName() == MethodHandleImpl.Intrinsic.NEW_ARRAY) {
            LambdaForm collectArgumentArrayForm = rebind.editor().collectArgumentArrayForm(1 + n, methodHandle2);
            if (collectArgumentArrayForm != null) {
                return rebind.copyWith(collectArgumentsChecks, collectArgumentArrayForm);
            }
        }
        return rebind.copyWithExtendL(collectArgumentsChecks, rebind.editor().collectArgumentsForm(1 + n, type.basicType()), methodHandle2);
    }
    
    private static MethodType collectArgumentsChecks(MethodHandle methodHandle, int n, MethodHandle methodHandle2) throws RuntimeException {
        MethodType type = methodHandle.type();
        MethodType type2 = methodHandle2.type();
        Class<Object> returnType = type2.returnType();
        List<Class<?>> parameterList = type2.parameterList();
        if (returnType == Void.TYPE) {
            return type.insertParameterTypes(n, parameterList);
        }
        if (returnType != type.parameterType(n)) {
            throw MethodHandleStatics.newIllegalArgumentException("target and filter types do not match", type, type2);
        }
        return type.dropParameterTypes(n, n + 1).insertParameterTypes(n, parameterList);
    }
    
    public static MethodHandle filterReturnValue(MethodHandle methodHandle, MethodHandle methodHandle2) {
        MethodType type = methodHandle.type();
        MethodType type2 = methodHandle2.type();
        filterReturnValueChecks(type, type2);
        BoundMethodHandle rebind = methodHandle.rebind();
        return rebind.copyWithExtendL(type.changeReturnType(type2.returnType()), rebind.editor().filterReturnForm(LambdaForm.BasicType.basicType(type2.returnType()), false), methodHandle2);
    }
    
    private static void filterReturnValueChecks(MethodType methodType, MethodType methodType2) throws RuntimeException {
        Class<Object> returnType = methodType.returnType();
        int parameterCount = methodType2.parameterCount();
        if ((parameterCount != 0) ? (returnType == methodType2.parameterType(0) && parameterCount == 1) : (returnType == Void.TYPE)) {
            return;
        }
        throw MethodHandleStatics.newIllegalArgumentException("target and filter types do not match", methodType, methodType2);
    }
    
    public static MethodHandle foldArguments(MethodHandle methodHandle, MethodHandle methodHandle2) {
        int n = 0;
        MethodType methodType = methodHandle.type();
        MethodType type = methodHandle2.type();
        Class<Object> foldArgumentChecks = foldArgumentChecks(n, methodType, type);
        BoundMethodHandle rebind = methodHandle.rebind();
        int n2 = foldArgumentChecks == Void.TYPE;
        LambdaForm foldArgumentsForm = rebind.editor().foldArgumentsForm(1 + n, (boolean)n2, type.basicType());
        if (n2 == 0) {
            methodType = methodType.dropParameterTypes(n, n + 1);
        }
        return rebind.copyWithExtendL(methodType, foldArgumentsForm, methodHandle2);
    }
    
    private static Class<?> foldArgumentChecks(int n, MethodType methodType, MethodType methodType2) {
        int parameterCount = methodType2.parameterCount();
        Class<Object> returnType = methodType2.returnType();
        int n2 = returnType != Void.TYPE;
        int n3 = n + n2;
        int n4 = methodType.parameterCount() >= n3 + parameterCount;
        if (n4 != 0 && !methodType2.parameterList().equals(methodType.parameterList().subList(n3, n3 + parameterCount))) {
            n4 = 0;
        }
        if (n4 != 0 && n2 != 0 && methodType2.returnType() != methodType.parameterType(0)) {
            n4 = 0;
        }
        if (n4 == 0) {
            throw misMatchedTypes("target and combiner types", methodType, methodType2);
        }
        return returnType;
    }
    
    public static MethodHandle guardWithTest(MethodHandle dropArguments, MethodHandle methodHandle, MethodHandle methodHandle2) {
        MethodType type = dropArguments.type();
        MethodType type2 = methodHandle.type();
        MethodType type3 = methodHandle2.type();
        if (!type2.equals(type3)) {
            throw misMatchedTypes("target and fallback types", type2, type3);
        }
        if (type.returnType() != Boolean.TYPE) {
            throw MethodHandleStatics.newIllegalArgumentException("guard type is not a predicate " + type);
        }
        List<Class<?>> parameterList = type2.parameterList();
        List<Class<?>> parameterList2 = type.parameterList();
        if (!parameterList.equals(parameterList2)) {
            int size = parameterList2.size();
            int size2 = parameterList.size();
            if (size >= size2 || !parameterList.subList(0, size).equals(parameterList2)) {
                throw misMatchedTypes("target and test types", type2, type);
            }
            dropArguments = dropArguments(dropArguments, size, parameterList.subList(size, size2));
            dropArguments.type();
        }
        return MethodHandleImpl.makeGuardWithTest(dropArguments, methodHandle, methodHandle2);
    }
    
    static RuntimeException misMatchedTypes(String str, MethodType obj, MethodType obj2) {
        return MethodHandleStatics.newIllegalArgumentException(str + " must match: " + obj + " != " + obj2);
    }
    
    public static MethodHandle catchException(MethodHandle methodHandle, Class<? extends Throwable> obj, MethodHandle dropArguments) {
        MethodType type = methodHandle.type();
        MethodType type2 = dropArguments.type();
        if (type2.parameterCount() < 1 || !type2.parameterType(0).isAssignableFrom(obj)) {
            throw MethodHandleStatics.newIllegalArgumentException("handler does not accept exception type " + obj);
        }
        if (type2.returnType() != type.returnType()) {
            throw misMatchedTypes("target and handler return types", type, type2);
        }
        List<Class<?>> parameterList = type.parameterList();
        List<Class<?>> parameterList2 = type2.parameterList();
        List<Object> subList = parameterList2.subList(1, parameterList2.size());
        if (!parameterList.equals(subList)) {
            int size = subList.size();
            int size2 = parameterList.size();
            if (size >= size2 || !parameterList.subList(0, size).equals(subList)) {
                throw misMatchedTypes("target and handler types", type, type2);
            }
            dropArguments = dropArguments(dropArguments, 1 + size, parameterList.subList(size, size2));
            dropArguments.type();
        }
        return MethodHandleImpl.makeGuardWithCatch(methodHandle, obj, dropArguments);
    }
    
    public static MethodHandle throwException(Class<?> clazz, Class<? extends Throwable> clazz2) {
        if (!Throwable.class.isAssignableFrom(clazz2)) {
            throw new ClassCastException(clazz2.getName());
        }
        return MethodHandleImpl.throwException(MethodType.methodType(clazz, clazz2));
    }
    
    static {
        IMPL_NAMES = MemberName.getFactory();
        MethodHandleImpl.initStatics();
        ACCESS_PERMISSION = new ReflectPermission("suppressAccessChecks");
        IDENTITY_MHS = new MethodHandle[Wrapper.values().length];
        ZERO_MHS = new MethodHandle[Wrapper.values().length];
    }
    
    public static final class Lookup
    {
        private final Class<?> lookupClass;
        private final int allowedModes;
        public static final int PUBLIC = 1;
        public static final int PRIVATE = 2;
        public static final int PROTECTED = 4;
        public static final int PACKAGE = 8;
        private static final int ALL_MODES = 15;
        private static final int TRUSTED = -1;
        static final MethodHandles.Lookup PUBLIC_LOOKUP;
        static final MethodHandles.Lookup IMPL_LOOKUP;
        private static final boolean ALLOW_NESTMATE_ACCESS = false;
        static ConcurrentHashMap<MemberName, DirectMethodHandle> LOOKASIDE_TABLE;
        static final /* synthetic */ boolean $assertionsDisabled;
        
        private static int fixmods(int n) {
            n &= 0x7;
            return (n != 0) ? n : 8;
        }
        
        public Class<?> lookupClass() {
            return this.lookupClass;
        }
        
        private Class<?> lookupClassOrNull() {
            return (this.allowedModes == -1) ? null : this.lookupClass;
        }
        
        public int lookupModes() {
            return this.allowedModes & 0xF;
        }
        
        Lookup(Class<?> clazz) {
            this(clazz, 15);
            checkUnprivilegedlookupClass(clazz, 15);
        }
        
        private Lookup(Class<?> lookupClass, int allowedModes) {
            this.lookupClass = lookupClass;
            this.allowedModes = allowedModes;
        }
        
        public MethodHandles.Lookup in(Class<?> clazz) {
            clazz.getClass();
            if (this.allowedModes == -1) {
                return new MethodHandles.Lookup(clazz, 15);
            }
            if (clazz == this.lookupClass) {
                return this;
            }
            int n = this.allowedModes & 0xB;
            if ((n & 0x8) != 0x0 && !VerifyAccess.isSamePackage(this.lookupClass, clazz)) {
                n &= 0xFFFFFFF5;
            }
            if ((n & 0x2) != 0x0 && !VerifyAccess.isSamePackageMember(this.lookupClass, clazz)) {
                n &= 0xFFFFFFFD;
            }
            if ((n & 0x1) != 0x0 && !VerifyAccess.isClassAccessible(clazz, this.lookupClass, this.allowedModes)) {
                n = 0;
            }
            checkUnprivilegedlookupClass(clazz, n);
            return new MethodHandles.Lookup(clazz, n);
        }
        
        private static void checkUnprivilegedlookupClass(Class<?> clazz, int n) {
            String name = clazz.getName();
            if (name.startsWith("java.lang.invoke.")) {
                throw MethodHandleStatics.newIllegalArgumentException("illegal lookupClass: " + clazz);
            }
            if (n == 15 && clazz.getClassLoader() == null && (name.startsWith("java.") || (name.startsWith("sun.") && !name.startsWith("sun.invoke.") && !name.equals("sun.reflect.ReflectionFactory")))) {
                throw MethodHandleStatics.newIllegalArgumentException("illegal lookupClass: " + clazz);
            }
            return;
        }
        
        @Override
        public String toString() {
            String name = this.lookupClass.getName();
            switch (this.allowedModes) {
                case 0: {
                    return name + "/noaccess";
                }
                case 1: {
                    return name + "/public";
                }
                case 9: {
                    return name + "/package";
                }
                case 11: {
                    return name + "/private";
                }
                case 15: {
                    return name;
                }
                case -1: {
                    return "/trusted";
                }
                default: {
                    String string = name + "/" + Integer.toHexString(this.allowedModes);
                    assert false : string;
                    return string;
                }
            }
        }
        
        public MethodHandle findStatic(Class<?> clazz, String s, MethodType methodType) throws NoSuchMethodException, IllegalAccessException {
            MemberName resolveOrFail = this.resolveOrFail((byte)6, clazz, s, methodType);
            return this.getDirectMethod((byte)6, clazz, resolveOrFail, this.findBoundCallerClass(resolveOrFail));
        }
        
        public MethodHandle findVirtual(Class<?> clazz, String s, MethodType methodType) throws NoSuchMethodException, IllegalAccessException {
            if (clazz == MethodHandle.class) {
                MethodHandle virtualForMH = this.findVirtualForMH(s, methodType);
                if (virtualForMH != null) {
                    return virtualForMH;
                }
            }
            int n = clazz.isInterface() ? 9 : 5;
            MemberName resolveOrFail = this.resolveOrFail((byte)n, clazz, s, methodType);
            return this.getDirectMethod((byte)n, clazz, resolveOrFail, this.findBoundCallerClass(resolveOrFail));
        }
        
        private MethodHandle findVirtualForMH(String s, MethodType methodType) {
            if ("invoke".equals(s)) {
                return MethodHandles.invoker(methodType);
            }
            if ("invokeExact".equals(s)) {
                return MethodHandles.exactInvoker(methodType);
            }
            assert !MemberName.isMethodHandleInvokeName(s);
            return null;
        }
        
        public MethodHandle findConstructor(Class<?> clazz, MethodType methodType) throws NoSuchMethodException, IllegalAccessException {
            if (clazz.isArray()) {
                throw new NoSuchMethodException("no constructor for array class: " + clazz.getName());
            }
            return this.getDirectConstructor(clazz, this.resolveOrFail((byte)8, clazz, "<init>", methodType));
        }
        
        public MethodHandle findSpecial(Class<?> clazz, String s, MethodType methodType, Class<?> clazz2) throws NoSuchMethodException, IllegalAccessException {
            this.checkSpecialCaller(clazz2);
            MethodHandles.Lookup in = this.in(clazz2);
            MemberName resolveOrFail = in.resolveOrFail((byte)7, clazz, s, methodType);
            return in.getDirectMethod((byte)7, clazz, resolveOrFail, this.findBoundCallerClass(resolveOrFail));
        }
        
        public MethodHandle findGetter(Class<?> clazz, String s, Class<?> clazz2) throws NoSuchFieldException, IllegalAccessException {
            return this.getDirectField((byte)1, clazz, this.resolveOrFail((byte)1, clazz, s, clazz2));
        }
        
        public MethodHandle findSetter(Class<?> clazz, String s, Class<?> clazz2) throws NoSuchFieldException, IllegalAccessException {
            return this.getDirectField((byte)3, clazz, this.resolveOrFail((byte)3, clazz, s, clazz2));
        }
        
        public MethodHandle findStaticGetter(Class<?> clazz, String s, Class<?> clazz2) throws NoSuchFieldException, IllegalAccessException {
            return this.getDirectField((byte)2, clazz, this.resolveOrFail((byte)2, clazz, s, clazz2));
        }
        
        public MethodHandle findStaticSetter(Class<?> clazz, String s, Class<?> clazz2) throws NoSuchFieldException, IllegalAccessException {
            return this.getDirectField((byte)4, clazz, this.resolveOrFail((byte)4, clazz, s, clazz2));
        }
        
        public MethodHandle bind(Object o, String s, MethodType methodType) throws NoSuchMethodException, IllegalAccessException {
            Class<Object> class1 = o.getClass();
            MemberName resolveOrFail = this.resolveOrFail((byte)7, class1, s, methodType);
            return this.getDirectMethodNoRestrict((byte)7, class1, resolveOrFail, this.findBoundCallerClass(resolveOrFail)).bindArgumentL(0, o).setVarargs(resolveOrFail);
        }
        
        public MethodHandle unreflect(Method method) throws IllegalAccessException {
            if (method.getDeclaringClass() == MethodHandle.class) {
                MethodHandle unreflectForMH = this.unreflectForMH(method);
                if (unreflectForMH != null) {
                    return unreflectForMH;
                }
            }
            MemberName o = new MemberName(method);
            int referenceKind = o.getReferenceKind();
            if (referenceKind == 7) {
                referenceKind = 5;
            }
            assert !(!o.isMethod());
            this = (method.isAccessible() ? MethodHandles.Lookup.IMPL_LOOKUP : this);
            return this.getDirectMethodNoSecurityManager((byte)referenceKind, o.getDeclaringClass(), o, this.findBoundCallerClass(o));
        }
        
        private MethodHandle unreflectForMH(Method method) {
            if (MemberName.isMethodHandleInvokeName(method.getName())) {
                return MethodHandleImpl.fakeMethodHandleInvoke(new MemberName(method));
            }
            return null;
        }
        
        public MethodHandle unreflectSpecial(Method method, Class<?> clazz) throws IllegalAccessException {
            this.checkSpecialCaller(clazz);
            MethodHandles.Lookup in = this.in(clazz);
            MemberName memberName = new MemberName(method, true);
            assert !(!memberName.isMethod());
            return in.getDirectMethodNoSecurityManager((byte)7, memberName.getDeclaringClass(), memberName, this.findBoundCallerClass(memberName));
        }
        
        public MethodHandle unreflectConstructor(Constructor<?> constructor) throws IllegalAccessException {
            MemberName memberName = new MemberName(constructor);
            assert !(!memberName.isConstructor());
            this = (constructor.isAccessible() ? MethodHandles.Lookup.IMPL_LOOKUP : this);
            return this.getDirectConstructorNoSecurityManager(memberName.getDeclaringClass(), memberName);
        }
        
        public MethodHandle unreflectGetter(Field field) throws IllegalAccessException {
            return this.unreflectField(field, false);
        }
        
        private MethodHandle unreflectField(Field field, boolean b) throws IllegalAccessException {
            MemberName memberName = new MemberName(field, b);
            if (!MethodHandles.Lookup.$assertionsDisabled) {
                if ((!b) ? MethodHandleNatives.refKindIsGetter(memberName.getReferenceKind()) : MethodHandleNatives.refKindIsSetter(memberName.getReferenceKind())) {}
                throw new AssertionError();
            }
            this = (field.isAccessible() ? MethodHandles.Lookup.IMPL_LOOKUP : this);
            return this.getDirectFieldNoSecurityManager(memberName.getReferenceKind(), field.getDeclaringClass(), memberName);
        }
        
        public MethodHandle unreflectSetter(Field field) throws IllegalAccessException {
            return this.unreflectField(field, true);
        }
        
        public MethodHandleInfo revealDirect(MethodHandle methodHandle) {
            MemberName internalMemberName = methodHandle.internalMemberName();
            if (internalMemberName == null || (!internalMemberName.isResolved() && !internalMemberName.isMethodHandleInvoke())) {
                throw MethodHandleStatics.newIllegalArgumentException("not a direct method handle");
            }
            Class<Object> declaringClass = internalMemberName.getDeclaringClass();
            int referenceKind = internalMemberName.getReferenceKind();
            assert !(!MethodHandleNatives.refKindIsValid(referenceKind));
            if (referenceKind == 7 && !methodHandle.isInvokeSpecial()) {
                referenceKind = 5;
            }
            if (referenceKind == 5 && declaringClass.isInterface()) {
                referenceKind = 9;
            }
            try {
                this.checkAccess((byte)referenceKind, declaringClass, internalMemberName);
                this.checkSecurityManager(declaringClass, internalMemberName);
            }
            catch (IllegalAccessException cause) {
                throw new IllegalArgumentException(cause);
            }
            if (this.allowedModes != -1 && internalMemberName.isCallerSensitive()) {
                Class<Object> internalCallerClass = methodHandle.internalCallerClass();
                if (!this.hasPrivateAccess() || internalCallerClass != this.lookupClass()) {
                    throw new IllegalArgumentException("method handle is caller sensitive: " + internalCallerClass);
                }
            }
            return new InfoFromMemberName(this, internalMemberName, (byte)referenceKind);
        }
        
        MemberName resolveOrFail(byte b, Class<?> clazz, String s, Class<?> clazz2) throws NoSuchFieldException, IllegalAccessException {
            this.checkSymbolicClass(clazz);
            s.getClass();
            clazz2.getClass();
            return MethodHandles.IMPL_NAMES.resolveOrFail(b, new MemberName(clazz, s, clazz2, b), this.lookupClassOrNull(), NoSuchFieldException.class);
        }
        
        MemberName resolveOrFail(byte b, Class<?> clazz, String s, MethodType methodType) throws NoSuchMethodException, IllegalAccessException {
            this.checkSymbolicClass(clazz);
            s.getClass();
            methodType.getClass();
            this.checkMethodName(b, s);
            return MethodHandles.IMPL_NAMES.resolveOrFail(b, new MemberName(clazz, s, methodType, b), this.lookupClassOrNull(), NoSuchMethodException.class);
        }
        
        MemberName resolveOrFail(byte b, MemberName memberName) throws ReflectiveOperationException {
            this.checkSymbolicClass(memberName.getDeclaringClass());
            memberName.getName().getClass();
            memberName.getType().getClass();
            return MethodHandles.IMPL_NAMES.resolveOrFail(b, memberName, this.lookupClassOrNull(), ReflectiveOperationException.class);
        }
        
        void checkSymbolicClass(Class<?> clazz) throws IllegalAccessException {
            clazz.getClass();
            Class<Object> lookupClassOrNull = this.lookupClassOrNull();
            if (lookupClassOrNull != null && !VerifyAccess.isClassAccessible(clazz, lookupClassOrNull, this.allowedModes)) {
                throw new MemberName(clazz).makeAccessException("symbolic reference class is not public", this);
            }
            return;
        }
        
        void checkMethodName(byte b, String str) throws NoSuchMethodException {
            if (str.startsWith("<") && b != 8) {
                throw new NoSuchMethodException("illegal method name: " + str);
            }
            return;
        }
        
        Class<?> findBoundCallerClass(MemberName memberName) throws IllegalAccessException {
            Class<Object> lookupClass = null;
            if (MethodHandleNatives.isCallerSensitive(memberName)) {
                if (this.hasPrivateAccess()) {
                    lookupClass = this.lookupClass;
                }
                else {
                    throw new IllegalAccessException("Attempt to lookup caller-sensitive method using restricted lookup object");
                }
            }
            return lookupClass;
        }
        
        private boolean hasPrivateAccess() {
            return (this.allowedModes & 0x2) != 0x0;
        }
        
        void checkSecurityManager(Class<?> clazz, MemberName memberName) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager == null) {
                return;
            }
            if (this.allowedModes == -1) {
                return;
            }
            int hasPrivateAccess = this.hasPrivateAccess();
            if (hasPrivateAccess == 0 || !VerifyAccess.classLoaderIsAncestor(this.lookupClass, clazz)) {
                ReflectUtil.checkPackageAccess(clazz);
            }
            if (memberName.isPublic()) {
                return;
            }
            if (hasPrivateAccess == 0) {
                securityManager.checkPermission(SecurityConstants.CHECK_MEMBER_ACCESS_PERMISSION);
            }
            Class<Object> declaringClass = memberName.getDeclaringClass();
            if (hasPrivateAccess == 0 && declaringClass != clazz) {
                ReflectUtil.checkPackageAccess(declaringClass);
            }
            return;
        }
        
        void checkMethod(byte b, Class<?> clazz, MemberName memberName) throws IllegalAccessException {
            int n = b == 6;
            String s;
            if (memberName.isConstructor()) {
                s = "expected a method, not a constructor";
            }
            else if (!memberName.isMethod()) {
                s = "expected a method";
            }
            else if (n != (memberName.isStatic() ? 1 : 0)) {
                s = (n ? "expected a static method" : "expected a non-static method");
            }
            else {
                this.checkAccess(b, clazz, memberName);
                return;
            }
            throw memberName.makeAccessException(s, this);
        }
        
        void checkField(byte b, Class<?> clazz, MemberName memberName) throws IllegalAccessException {
            int n = !MethodHandleNatives.refKindHasReceiver(b);
            if (n != (memberName.isStatic() ? 1 : 0)) {
                throw memberName.makeAccessException(n ? "expected a static field" : "expected a non-static field", this);
            }
            this.checkAccess(b, clazz, memberName);
            return;
        }
        
        void checkAccess(byte b, Class<?> clazz, MemberName memberName) throws IllegalAccessException {
            assert memberName.referenceKindIsConsistentWith((int)b) && MethodHandleNatives.refKindIsValid((int)b) && MethodHandleNatives.refKindIsField(b) == memberName.isField();
            int allowedModes = this.allowedModes;
            if (allowedModes == -1) {
                return;
            }
            int modifiers = memberName.getModifiers();
            if (Modifier.isProtected(modifiers) && b == 5 && memberName.getDeclaringClass() == Object.class && memberName.getName().equals("clone") && clazz.isArray()) {
                modifiers ^= 0x5;
            }
            if (Modifier.isProtected(modifiers) && b == 8) {
                modifiers ^= 0x4;
            }
            if (Modifier.isFinal(modifiers) && MethodHandleNatives.refKindIsSetter(b)) {
                throw memberName.makeAccessException("unexpected set of a final field", this);
            }
            if (Modifier.isPublic(modifiers) && Modifier.isPublic(clazz.getModifiers()) && allowedModes != 0) {
                return;
            }
            int fixmods = fixmods(modifiers);
            if ((fixmods & allowedModes) != 0x0) {
                if (VerifyAccess.isMemberAccessible(clazz, memberName.getDeclaringClass(), modifiers, this.lookupClass(), allowedModes)) {
                    return;
                }
            }
            else if ((fixmods & 0x4) != 0x0 && (allowedModes & 0x8) != 0x0 && VerifyAccess.isSamePackage(memberName.getDeclaringClass(), this.lookupClass())) {
                return;
            }
            throw memberName.makeAccessException(this.accessFailedMessage(clazz, memberName), this);
        }
        
        String accessFailedMessage(Class<?> clazz, MemberName memberName) {
            Class<Object> declaringClass = memberName.getDeclaringClass();
            int modifiers = memberName.getModifiers();
            int n = Modifier.isPublic(declaringClass.getModifiers()) && (declaringClass == clazz || Modifier.isPublic(clazz.getModifiers()));
            if (n == 0 && (this.allowedModes & 0x8) != 0x0) {
                n = (VerifyAccess.isClassAccessible(declaringClass, this.lookupClass(), 15) && (declaringClass == clazz || VerifyAccess.isClassAccessible(clazz, this.lookupClass(), 15)));
            }
            if (n == 0) {
                return "class is not public";
            }
            if (Modifier.isPublic(modifiers)) {
                return "access to public member failed";
            }
            if (Modifier.isPrivate(modifiers)) {
                return "member is private";
            }
            if (Modifier.isProtected(modifiers)) {
                return "member is protected";
            }
            return "member is private to package";
        }
        
        private void checkSpecialCaller(Class<?> clazz) throws IllegalAccessException {
            if (this.allowedModes == -1) {
                return;
            }
            if (!this.hasPrivateAccess() || clazz != this.lookupClass()) {
                throw new MemberName(clazz).makeAccessException("no private access for invokespecial", this);
            }
            return;
        }
        
        private boolean restrictProtectedReceiver(MemberName memberName) {
            return memberName.isProtected() && !memberName.isStatic() && this.allowedModes != -1 && memberName.getDeclaringClass() != this.lookupClass() && !VerifyAccess.isSamePackage(memberName.getDeclaringClass(), this.lookupClass());
        }
        
        private MethodHandle restrictReceiver(MemberName memberName, DirectMethodHandle directMethodHandle, Class<?> clazz) throws IllegalAccessException {
            assert !memberName.isStatic();
            if (!memberName.getDeclaringClass().isAssignableFrom(clazz)) {
                throw memberName.makeAccessException("caller class must be a subclass below the method", clazz);
            }
            MethodType type = directMethodHandle.type();
            if (type.parameterType(0) == clazz) {
                return directMethodHandle;
            }
            MethodType changeParameterType = type.changeParameterType(0, clazz);
            assert !directMethodHandle.isVarargsCollector();
            assert !(!directMethodHandle.viewAsTypeChecks(changeParameterType, true));
            return directMethodHandle.copyWith(changeParameterType, directMethodHandle.form);
        }
        
        private MethodHandle getDirectMethod(byte b, Class<?> clazz, MemberName memberName, Class<?> clazz2) throws IllegalAccessException {
            return this.getDirectMethodCommon(b, clazz, memberName, true, true, clazz2);
        }
        
        private MethodHandle getDirectMethodNoRestrict(byte b, Class<?> clazz, MemberName memberName, Class<?> clazz2) throws IllegalAccessException {
            return this.getDirectMethodCommon(b, clazz, memberName, true, false, clazz2);
        }
        
        private MethodHandle getDirectMethodNoSecurityManager(byte b, Class<?> clazz, MemberName memberName, Class<?> clazz2) throws IllegalAccessException {
            return this.getDirectMethodCommon(b, clazz, memberName, false, true, clazz2);
        }
        
        private MethodHandle getDirectMethodCommon(byte b, Class<?> clazz, MemberName varargs, boolean b2, boolean b3, Class<?> clazz2) throws IllegalAccessException {
            this.checkMethod(b, clazz, varargs);
            if (b2) {
                this.checkSecurityManager(clazz, varargs);
            }
            assert !varargs.isMethodHandleInvoke();
            if (b == 7 && clazz != this.lookupClass() && !clazz.isInterface() && clazz != this.lookupClass().getSuperclass() && clazz.isAssignableFrom(this.lookupClass())) {
                assert !varargs.getName().equals("<init>");
                Class<Object> clazz3 = this.lookupClass();
                MemberName resolveOrNull;
                do {
                    clazz3 = clazz3.getSuperclass();
                    resolveOrNull = MethodHandles.IMPL_NAMES.resolveOrNull(b, new MemberName(clazz3, varargs.getName(), varargs.getMethodType(), (byte)7), this.lookupClassOrNull());
                } while (resolveOrNull == null && clazz != clazz3);
                if (resolveOrNull == null) {
                    throw new InternalError(varargs.toString());
                }
                varargs = resolveOrNull;
                clazz = clazz3;
                this.checkMethod(b, clazz, varargs);
            }
            DirectMethodHandle directMethodHandle = DirectMethodHandle.make(b, clazz, varargs);
            if (b3 && (b == 7 || (MethodHandleNatives.refKindHasReceiver(b) && this.restrictProtectedReceiver(varargs)))) {
                directMethodHandle = this.restrictReceiver(varargs, directMethodHandle, this.lookupClass());
            }
            return this.maybeBindCaller(varargs, directMethodHandle, clazz2).setVarargs(varargs);
        }
        
        private MethodHandle maybeBindCaller(MemberName memberName, MethodHandle methodHandle, Class<?> clazz) throws IllegalAccessException {
            if (this.allowedModes == -1 || !MethodHandleNatives.isCallerSensitive(memberName)) {
                return methodHandle;
            }
            Class<Object> lookupClass = this.lookupClass;
            if (!this.hasPrivateAccess()) {
                lookupClass = clazz;
            }
            return MethodHandleImpl.bindCaller(methodHandle, lookupClass);
        }
        
        private MethodHandle getDirectField(byte b, Class<?> clazz, MemberName memberName) throws IllegalAccessException {
            return this.getDirectFieldCommon(b, clazz, memberName, true);
        }
        
        private MethodHandle getDirectFieldNoSecurityManager(byte b, Class<?> clazz, MemberName memberName) throws IllegalAccessException {
            return this.getDirectFieldCommon(b, clazz, memberName, false);
        }
        
        private MethodHandle getDirectFieldCommon(byte b, Class<?> clazz, MemberName memberName, boolean b2) throws IllegalAccessException {
            this.checkField(b, clazz, memberName);
            if (b2) {
                this.checkSecurityManager(clazz, memberName);
            }
            DirectMethodHandle make = DirectMethodHandle.make(clazz, memberName);
            if (MethodHandleNatives.refKindHasReceiver(b) && this.restrictProtectedReceiver(memberName)) {
                return this.restrictReceiver(memberName, make, this.lookupClass());
            }
            return make;
        }
        
        private MethodHandle getDirectConstructor(Class<?> clazz, MemberName memberName) throws IllegalAccessException {
            return this.getDirectConstructorCommon(clazz, memberName, true);
        }
        
        private MethodHandle getDirectConstructorNoSecurityManager(Class<?> clazz, MemberName memberName) throws IllegalAccessException {
            return this.getDirectConstructorCommon(clazz, memberName, false);
        }
        
        private MethodHandle getDirectConstructorCommon(Class<?> clazz, MemberName varargs, boolean b) throws IllegalAccessException {
            assert !(!varargs.isConstructor());
            this.checkAccess((byte)8, clazz, varargs);
            if (b) {
                this.checkSecurityManager(clazz, varargs);
            }
            assert !MethodHandleNatives.isCallerSensitive(varargs);
            return DirectMethodHandle.make(varargs).setVarargs(varargs);
        }
        
        MethodHandle linkMethodHandleConstant(byte b, Class<?> clazz, String s, Object o) throws ReflectiveOperationException {
            if (!(o instanceof Class) && !(o instanceof MethodType)) {
                throw new InternalError("unresolved MemberName");
            }
            MemberName key = new MemberName(b, clazz, s, o);
            MethodHandle methodHandle = (MethodHandle)MethodHandles.Lookup.LOOKASIDE_TABLE.get(key);
            if (methodHandle != null) {
                this.checkSymbolicClass(clazz);
                return methodHandle;
            }
            if (clazz == MethodHandle.class && b == 5) {
                MethodHandle virtualForMH = this.findVirtualForMH(key.getName(), key.getMethodType());
                if (virtualForMH != null) {
                    return virtualForMH;
                }
            }
            MemberName resolveOrFail = this.resolveOrFail(b, key);
            MethodHandle directMethodForConstant = this.getDirectMethodForConstant(b, clazz, resolveOrFail);
            if (directMethodForConstant instanceof DirectMethodHandle && this.canBeCached(b, clazz, resolveOrFail)) {
                MemberName key2 = directMethodForConstant.internalMemberName();
                if (key2 != null) {
                    key2 = key2.asNormalOriginal();
                }
                if (key.equals(key2)) {
                    MethodHandles.Lookup.LOOKASIDE_TABLE.put(key2, (DirectMethodHandle)directMethodForConstant);
                }
            }
            return directMethodForConstant;
        }
        
        private boolean canBeCached(byte b, Class<?> clazz, MemberName memberName) {
            if (b == 7) {
                return false;
            }
            if (!Modifier.isPublic(clazz.getModifiers()) || !Modifier.isPublic(memberName.getDeclaringClass().getModifiers()) || !memberName.isPublic() || memberName.isCallerSensitive()) {
                return false;
            }
            ClassLoader classLoader = clazz.getClassLoader();
            if (!VM.isSystemDomainLoader(classLoader)) {
                ClassLoader classLoader2 = ClassLoader.getSystemClassLoader();
                boolean n = false;
                while (classLoader2 != null) {
                    if (classLoader == classLoader2) {
                        n = true;
                        break;
                    }
                    classLoader2 = classLoader2.getParent();
                }
                if (n == 0) {
                    return false;
                }
            }
            try {
                this.checkSecurityManager(clazz, MethodHandles.publicLookup().resolveOrFail(b, new MemberName(b, clazz, memberName.getName(), memberName.getType())));
            }
            catch (ReflectiveOperationException | SecurityException ex) {
                return false;
            }
            return true;
        }
        
        private MethodHandle getDirectMethodForConstant(byte b, Class<?> clazz, MemberName obj) throws ReflectiveOperationException {
            if (MethodHandleNatives.refKindIsField(b)) {
                return this.getDirectFieldNoSecurityManager(b, clazz, obj);
            }
            if (MethodHandleNatives.refKindIsMethod(b)) {
                return this.getDirectMethodNoSecurityManager(b, clazz, obj, this.lookupClass);
            }
            if (b == 8) {
                return this.getDirectConstructorNoSecurityManager(clazz, obj);
            }
            throw MethodHandleStatics.newIllegalArgumentException("bad MethodHandle constant #" + obj);
        }
        
        static {
            MethodHandles.IMPL_NAMES.getClass();
            PUBLIC_LOOKUP = new MethodHandles.Lookup(Object.class, 1);
            IMPL_LOOKUP = new MethodHandles.Lookup(Object.class, -1);
            MethodHandles.Lookup.LOOKASIDE_TABLE = new ConcurrentHashMap();
        }
    }
}
